# include "hbdisplay.h"

# include <harfbuzz/hb.h>
# include <harfbuzz/hb-ft.h>
# include <harfbuzz/hb-glib.h>

# include <QtGui>

// adapted from https://github.com/khaledhosny/QtHarfBuzzExample.git
void HBDisplay :: paintEvent ( QPaintEvent * )
{
	if ( not QFile ( fileName ) . exists () ) return ;
	QPainter painter ( this ) ;

	qreal marginSize = 10.0 ;
	qreal fontScale = 64.0 ;

	// prepare FreeType font
	FT_Library ftLibrary ;
	FT_Init_FreeType ( & ftLibrary ) ;

	FT_Face ftFace ;
	FT_New_Face ( ftLibrary, fileName . toUtf8 () . constData (), 0, & ftFace ) ;
	FT_Set_Pixel_Sizes ( ftFace, 0, fontSize ) ;

	// prepare HarfBuzz font and buffer
	hb_font_t * hbFont = hb_ft_font_create ( ftFace, NULL ) ;
	hb_buffer_t * hbBuffer = hb_buffer_create () ;

	// set text and shape the buffer
	hb_buffer_add_utf8 ( hbBuffer, text . toUtf8 () . constData (), -1, 0, -1 ) ;
	hb_shape ( hbFont, hbBuffer, NULL, 0 ) ;

	// extract shaped glyphs from the buffer
	int nGlyphs = hb_buffer_get_length ( hbBuffer ) ;
	hb_glyph_info_t * hbGlyphs = hb_buffer_get_glyph_infos ( hbBuffer, NULL ) ;
	hb_glyph_position_t * hbPositions = hb_buffer_get_glyph_positions ( hbBuffer, NULL ) ;

	// map shaped glyph indices and position for Qt's glyph run
	QVector < quint32 > glyphIndexes   ( nGlyphs ) ;
	QVector < QPointF > glyphPositions ( nGlyphs ) ;

	qreal x = 0.0, y = 0.0;
	for ( int i = 0 ; i < nGlyphs ; i ++, hbGlyphs ++, hbPositions ++ )
	{
		glyphIndexes   [ i ] = hbGlyphs -> codepoint ;
		glyphPositions [ i ] = QPointF ( x + hbPositions -> x_offset, y - hbPositions -> y_offset ) / fontScale ;
		x += hbPositions -> x_advance ;
		y -= hbPositions -> y_advance ;
	}

	// prepare Qt glyph run and raw font to draw the glyphs
	QRawFont rawFont = QRawFont ( fileName, fontSize ) ;
	QGlyphRun glyphRun = QGlyphRun () ;
	glyphRun . setRawFont ( rawFont );
	glyphRun . setGlyphIndexes ( glyphIndexes ) ;
	glyphRun . setPositions ( glyphPositions ) ;

	// draw the glyphs to the painter
	painter . drawGlyphRun ( QPointF ( marginSize, rawFont . ascent () + marginSize ), glyphRun ) ;

	// fit the widget to the drawn glyphs
	resize ( x / fontScale + marginSize * 2, rawFont . ascent () + rawFont . descent () + marginSize * 2 ) ;

	hb_buffer_destroy ( hbBuffer ) ;
	hb_font_destroy ( hbFont ) ;
	FT_Done_FreeType ( ftLibrary ) ;
}
